#!/usr/bin/python
# -*- coding: utf-8 -*-

###########################################
# IMPORT LIBRARY
###########################################

import numpy as np
from numpy import linalg as LA
from scipy import *
#import matplotlib.pyplot as plt


#Retourne l'indice de la case m en fonction des indices (i,j) (0 <= i,j <= N-1) 
def getNumCase(i,j,N) :
   return i*N+j;

#Retourne l'indice k de la fonction de forme \psi_k(x,y) = \phi_i(x)\phi_j(y) en fonction des indices (i,j). 
def getInd(i,j,N) :

	if ((i>=1) & (i<=N-1) & (j>=1) & (j<=N-1)) : 
		indi = i-1; 
		indj = j-1; 

		return (N-1)*indi + indj;

	else :
		return -1; 


#Convertit un vecteur de taille (N-1)*(N-1) en une matrice de taille (N-1) * (N+1)
def convertToMat(U,N):

	V = np.zeros((N-1,N-1)); 
	for i in range(0,N-1):
		for j in range(0,N-1):
			V[i,j] = U[getInd(i+1,j+1,N)]; 

	return V;

#Convertit un vecteur de taille (N)*(N) en une matrice de taille (N) * (N)
def convertToMat2(U,N):

	V = np.zeros((N,N)); 
	for i in range(0,N):
		for j in range(0,N):
			V[i,j] = U[getNumCase(i,j,N)]; 

	return V;


 
# C doit être un vecteur de taille N*N contenant les valeurs du coefficient de diffusion c(x,y) dans les cases
# Renvoie la matrice contenant les éléments B_{kl} = \int_D c(x,y) \partial_x \psi_k(x,y) \partial_x \psi_l(x,y)
def getMatdxdx(C,N): 
	K = (N-1)*(N-1); 	
	B = np.zeros((K,K));

	dx = 1.0/N; 

	for i in range(1,N):
		
		for j in range(1,N):

			indij = getInd(i,j,N); 

			# Case (i,j)
			m = getNumCase(i,j,N); 
			B[indij, indij] += C[m] * (1.0/dx) * (dx/3.0);  

			indipj = getInd(i+1,j,N);
                    	if (indipj >-0.5):
				B[indij, indipj] += C[m] * (-1.0/dx) * (dx/3.0); 
                    
			indipjp = getInd(i+1,j+1,N);
                    	if (indipjp >-0.5):
				 B[indij, indipjp] += C[m] * (-1.0/dx) * (dx/6.0);
                    
                  	indijp = getInd(i,j+1,N);
                    	if (indijp >-0.5):
				 B[indij, indijp] += C[m] * (1.0/dx) * (dx/6.0);



                
                	# Case (i-1,j)
                	m = getNumCase(i-1,j,N); 
                 	B[indij, indij] += C[m] * (1.0/dx) * (dx/3.0); 
                    
                    	indipj = getInd(i-1,j,N);
                    	if (indipj >-0.5):
				B[indij, indipj] += C[m] * (-1.0/dx) * (dx/3.0); 
                    
                    	indipjp = getInd(i-1,j+1,N);
                    	if (indipjp >-0.5):
				B[indij, indipjp] += C[m] * (-1.0/dx) * (dx/6.0);
                    
                    	indijp = getInd(i,j+1,N);
                    	if (indijp >-0.5):
				B[indij, indijp] += C[m] * (1.0/dx) * (dx/6.0);
                    

                
                	#Case (i-1,j-1)
                	m = getNumCase(i-1,j-1,N); 
                    	B[indij, indij] += C[m] * (1.0/dx) * (dx/3.0); 
                    
                    	indipj = getInd(i-1,j,N);
                    	if (indipj >-0.5):
				B[indij, indipj] += C[m] * (-1.0/dx) * (dx/3.0); 
                    
                    	indipjp = getInd(i-1,j-1,N);
                    	if (indipjp >-0.5):
				B[indij, indipjp] += C[m] * (-1.0/dx) * (dx/6.0);
                    
                    	indijp = getInd(i,j-1,N);
                    	if (indijp >-0.5):
				B[indij, indijp] += C[m] * (1.0/dx) * (dx/6.0);
                    
                
                	#Case (i,j-1)
                	m = getNumCase(i,j-1,N); 
                   	B[indij, indij] += C[m] * (1.0/dx) * (dx/3.0); 
                    
                    	indipj = getInd(i+1,j,N);
                    	if (indipj >-0.5):
				B[indij, indipj] += C[m] * (-1.0/dx) * (dx/3.0); 
                    
                    	indipjp = getInd(i+1,j-1,N);
                    	if (indipjp >-0.5):
				B[indij, indipjp] += C[m] * (-1.0/dx) * (dx/6.0);
                    
                    	indijp = getInd(i,j-1,N);
                    	if (indijp >-0.5):
				B[indij, indijp] += C[m] * (1.0/dx) * (dx/6.0);
                
	return B;


# C doit être un vecteur de taille N*N contenant les valeurs du coefficient de diffusion c(x,y) dans les cases
# Renvoie la matrice contenant les éléments B_{kl} = \int_D c(x,y) \partial_y \psi_k(x,y) \partial_y \psi_l(x,y)
def getMatdydy(C,N): 
	K = (N-1)*(N-1); 	
	B = np.zeros((K,K));

	dx = 1.0/N; 

	for i in range(1,N):
		
		for j in range(1,N):

			indij = getInd(i,j,N); 

			# Case (i,j)
			m = getNumCase(i,j,N); 
			B[indij, indij] += C[m] * (dx/3.0) * (1.0/dx);  

			indipj = getInd(i+1,j,N);
                    	if (indipj >-0.5):
				B[indij, indipj] += C[m] * (dx/6.0) * (1.0/dx); 
                    
			indipjp = getInd(i+1,j+1,N);
                    	if (indipjp >-0.5):
				 B[indij, indipjp] += C[m] * (dx/6.0) * (-1.0/dx);
                    
                  	indijp = getInd(i,j+1,N);
                    	if (indijp >-0.5):
				 B[indij, indijp] += C[m] * (dx/3.0) * (-1.0/dx);



                
                	# Case (i-1,j)
                	m = getNumCase(i-1,j,N); 
                 	B[indij, indij] += C[m] * (dx/3.0) * (1.0/dx); 
                    
                    	indipj = getInd(i-1,j,N);
                    	if (indipj >-0.5):
				B[indij, indipj] += C[m] * (dx/6.0) * (1.0/dx); 
                    
                    	indipjp = getInd(i-1,j+1,N);
                    	if (indipjp >-0.5):
				B[indij, indipjp] += C[m] * (dx/6.0) * (-1.0/dx);
                    
                    	indijp = getInd(i,j+1,N);
                    	if (indijp >-0.5):
				B[indij, indijp] += C[m] * (dx/3.0) * (-1.0/dx);
                    

                
                	#Case (i-1,j-1)
                	m = getNumCase(i-1,j-1,N); 
                    	B[indij, indij] += C[m] * (dx/3.0) * (1.0/dx); 
                    
                    	indipj = getInd(i-1,j,N);
                    	if (indipj >-0.5):
				B[indij, indipj] += C[m] * (dx/6.0) * (1.0/dx); 
                    
                    	indipjp = getInd(i-1,j-1,N);
                    	if (indipjp >-0.5):
				B[indij, indipjp] += C[m] * (dx/6.0) * (-1.0/dx);
                    
                    	indijp = getInd(i,j-1,N);
                    	if (indijp >-0.5):
				B[indij, indijp] += C[m] * (dx/3.0) * (-1.0/dx);
                    
                
                	#Case (i,j-1)
                	m = getNumCase(i,j-1,N); 
                   	B[indij, indij] += C[m] * (dx/3.0) * (1.0/dx); 
                    
                    	indipj = getInd(i+1,j,N);
                    	if (indipj >-0.5):
				B[indij, indipj] += C[m] * (dx/6.0) * (1.0/dx); 
                    
                    	indipjp = getInd(i+1,j-1,N);
                    	if (indipjp >-0.5):
				B[indij, indipjp] += C[m] * (dx/6.0) * (-1.0/dx);
                    
                    	indijp = getInd(i,j-1,N);
                    	if (indijp >-0.5):
				B[indij, indijp] += C[m] * (dx/3.0) * (-1.0/dx);
                
	return B;


# C doit être un vecteur de taille N*N contenant les valeurs du coefficient de diffusion c(x,y) dans les cases
# Renvoie la matrice contenant les éléments B_{kl} = \int_D c(x,y)\psi_k(x,y)\psi_l(x,y)
def getMatmass(C,N): 
	K = (N-1)*(N-1); 	
	B = np.zeros((K,K));

	dx = 1.0/N; 

	for i in range(1,N):
		
		for j in range(1,N):

			indij = getInd(i,j,N); 

			# Case (i,j)
			m = getNumCase(i,j,N); 
			B[indij, indij] += C[m] * (dx/3.0) * (dx/3.0);  

			indipj = getInd(i+1,j,N);
                    	if (indipj >-0.5):
				B[indij, indipj] += C[m] * (dx/6.0) * (dx/3.0); 
                    
			indipjp = getInd(i+1,j+1,N);
                    	if (indipjp >-0.5):
				 B[indij, indipjp] += C[m] * (dx/6.0) * (dx/6.0);
                    
                  	indijp = getInd(i,j+1,N);
                    	if (indijp >-0.5):
				 B[indij, indijp] += C[m] * (dx/3.0) * (dx/6.0);



                
                	# Case (i-1,j)
                	m = getNumCase(i-1,j,N); 
                 	B[indij, indij] += C[m] * (dx/3.0) * (dx/3.0); 
                    
                    	indipj = getInd(i-1,j,N);
                    	if (indipj >-0.5):
				B[indij, indipj] += C[m] * (dx/6.0) * (dx/3.0); 
                    
                    	indipjp = getInd(i-1,j+1,N);
                    	if (indipjp >-0.5):
				B[indij, indipjp] += C[m] * (dx/6.0) * (dx/6.0);
                    
                    	indijp = getInd(i,j+1,N);
                    	if (indijp >-0.5):
				B[indij, indijp] += C[m] * (dx/3.0) * (dx/6.0);
                    

                
                	#Case (i-1,j-1)
                	m = getNumCase(i-1,j-1,N); 
                    	B[indij, indij] += C[m] * (dx/3.0) * (dx/3.0); 
                    
                    	indipj = getInd(i-1,j,N);
                    	if (indipj >-0.5):
				B[indij, indipj] += C[m] * (dx/6.0) * (dx/3.0); 
                    
                    	indipjp = getInd(i-1,j-1,N);
                    	if (indipjp >-0.5):
				B[indij, indipjp] += C[m] * (dx/6.0) * (dx/6.0);
                    
                    	indijp = getInd(i,j-1,N);
                    	if (indijp >-0.5):
				B[indij, indijp] += C[m] * (dx/3.0) * (dx/6.0);
                    
                
                	#Case (i,j-1)
                	m = getNumCase(i,j-1,N); 
                   	B[indij, indij] += C[m] * (dx/3.0) * (dx/3.0); 
                    
                    	indipj = getInd(i+1,j,N);
                    	if (indipj >-0.5):
				B[indij, indipj] += C[m] * (dx/6.0) * (dx/3.0); 
                    
                    	indipjp = getInd(i+1,j-1,N);
                    	if (indipjp >-0.5):
				B[indij, indipjp] += C[m] * (dx/6.0) * (dx/6.0);
                    
                    	indijp = getInd(i,j-1,N);
                    	if (indijp >-0.5):
				B[indij, indijp] += C[m] * (dx/3.0) * (dx/6.0);
                
	return B;
