#!/usr/bin/python
# -*- coding: utf-8 -*-

###########################################
# IMPORT LIBRARY
###########################################

import numpy as np
from numpy import linalg as LA
from scipy import *
#import matplotlib.pyplot as plt


################################################################
#
# Assemblage des matrices discrétisées dont nous aurons besoin
#
#################################################################


#Retourne l'indice de la fonction de forme  \phi_i(x), fonction chapeau centrée en i. 
def getInd(i,N):

	if ( (i>=1) & (i<=(N-1)) ) : 
		return i-1; 
	else :
		return -1;  



# C doit être un vecteur de taille N contenant les valeurs du coefficient de diffusion c(x) dans les différentes cases de la subdivision
# C_i (1<= i <=N) correspond à la valeur du coefficient c(x) dans (x_{i-1}, x_i)
# Renvoie la matrice contenant les éléments B_{ij} = \int_(0,1) c(x) \partial_x \phi_i(x) \partial_x \phi_j(x) dx
def getMatdxdx(C,N): 
	K = (N-1); 	
	B = np.zeros((K,K));

	dx = 1.0/N; 

	for i in range(1,N):

		indi = getInd(i,N); 

		# Case (i)
		m = i; 
		B[indi, indi] += C[m] * (1.0/dx);  

		indip = getInd(i+1,N);
                if (indip >-0.5):
			B[indi, indip] += C[m] * (-1.0/dx) ; 
                    
                # Case (i-1)
                m = i-1; 
                B[indi, indi] += C[m] * (1.0/dx); 
                    
                indip = getInd(i-1,N);
                if (indip >-0.5):
			B[indi, indip] += C[m] * (-1.0/dx); 

	return B;


# C doit être un vecteur de taille N contenant les valeurs du coefficient de diffusion c(x) dans les différentes cases de la subdivision
# C_i (1<= i <=N) correspond à la valeur du coefficient c(x) dans (x_{i-1}, x_i)
# Renvoie la matrice contenant les éléments B_{ij} = \int_(0,1) c(x)\phi_i(x) \phi_j(x) dx
def getMatmass(C,N): 
	K = (N-1); 	
	B = np.zeros((K,K));

	dx = 1.0/N; 

	for i in range(1,N):
		
		indi = getInd(i,N); 

		# Case (i)
		m = i; 
		B[indi, indi] += C[m] * (dx/3.0);  

		indip = getInd(i+1,N);
                if (indip >-0.5):
			B[indi, indip] += C[m] * (dx/6.0); 
                    
		# Case (i-1)
               	m = i-1; 
                B[indi, indi] += C[m] * (dx/3.0); 
                    
                indip = getInd(i-1,N);
                if (indip >-0.5):
			B[indi, indip] += C[m] * (dx/6.0); 
                    
                
	return B;



## Fonction qui transforme un vecteur U de taille N-1 correspondant aux paramètres de discrétisation d'une grille grossière 
## en un vecteur Uref de taille Nref-1 correspondant aux paramètres de discrétisation d'une grille fine
## Il faut que Nref = 2^p * N pour un certain entier p
def convert(U,N,Nref):

	K = U.shape; 
	k = K[1]; 
	M = 2*N;
	Utemp = np.zeros((N+1,k)); 
	Utemp[1:N,:] = U; 
	Ntemp = N;
	while (M <= Nref):
		Unew = np.zeros((M-1,k)); 

		for i in range(0,Ntemp-1):
			Unew[2*i,:] = 0.5*(Utemp[i,:] + Utemp[i+1,:]); 
			Unew[2*i+1,:] = Utemp[i+1,:]; 
		Unew[2*Ntemp-2,:] = 0.5*(Utemp[Ntemp-1,:] + Utemp[Ntemp,:]);

		Utemp = np.zeros((M+1,k)); 
 		Utemp[1:M,:] = Unew; 
		Ntemp = 2*Ntemp; 
		M = 2*M; 

	return Unew; 



 
